// A combination of a few things:
// * Recursive drawing of a Sierpinski gasket
// * Drawing based on explicit triangle coordinates and
//   midpoints instead of geometric context.
// * Using direct manipulation to control the outermost
//   triangle

// Number of direct manipulation points, as in Module 03.
int num_dots = 3;

float[] hxs;
float[] hys;

int which_drag;
float radius = 8.0;

// Number of recursive levels.
int levels = 8;

void setup()
{
  size( 500, 500 );
  float h = 20 + 230 * sqrt(3);
  
  hxs = new float[] { 20, 250, 480 };
  hys = new float[] { h, 20, h };

  which_drag = -1;
}

void drawTriangle(
  int levs, 
  float ax, float ay,
  float bx, float by,
  float cx, float cy )
{
  if( levs == 0 ) {
    // Base case -- no more recursive levels to draw.
    // Just draw a triangle.
    
    triangle( ax, ay, bx, by, cx, cy );
  } else {
    // Recursive case -- draw three recursive copies of
    // the gasket at the next level down.  Compute the
    // triangles bounding those recursive copies by computing
    // midpoints of the three points of the passed-in
    // triangle.
    
    float mabx = lerp( ax, bx, 0.5 );
    float maby = lerp( ay, by, 0.5 );
    float macx = lerp( ax, cx, 0.5 );
    float macy = lerp( ay, cy, 0.5 );
    float mbcx = lerp( bx, cx, 0.5 );
    float mbcy = lerp( by, cy, 0.5 );
    
    drawTriangle( levs - 1, ax, ay, mabx, maby, macx, macy );
    drawTriangle( levs - 1, mabx, maby, bx, by, mbcx, mbcy );
    drawTriangle( levs - 1, macx, macy, mbcx, mbcy, cx, cy );
  }
}

void draw()
{
  background( 255 );
  
  // Draw the fractal...
  noStroke();
  fill( 0 );
  drawTriangle( levels, 
    hxs[0], hys[0], hxs[1], hys[1], hxs[2], hys[2] );
  
  // ...and the handles for direct manipulation.
  for ( int idx = 0; idx < num_dots; ++idx ) {
    if ( which_drag == idx ) {
      stroke( 255, 0, 0 );
    } else {
      stroke( 0, 0, 0 );
    }
    fill( 128 );
    ellipse( hxs[idx], hys[idx], 2.0*radius, 2.0*radius );
  }
}

void mousePressed()
{
  for ( int idx = 0; idx < num_dots; ++idx ) {
    float dx = mouseX - hxs[idx];
    float dy = mouseY - hys[idx];
    if ( sqrt(sq(dx) + sq(dy)) <= radius ) {
      which_drag = idx;
    }
  }
}

void mouseDragged()
{
  if ( which_drag != -1 ) {
    hxs[which_drag] = mouseX;
    hys[which_drag] = mouseY;
  }
}

void mouseReleased()
{
  which_drag = -1;
}

