// Use the Gray-Scott model to simulate the reaction-diffusion
// equations.  This is actually fairly advanced material, so
// I won't try too hard to explain how it works.  Let's just
// say that it's a kind of physical simulation constrained to
// a grid.  It won't always work (it's sensitive to the starting
// grid, so sometimes you'll get an all-black or all-white screen).
// But when it works you'll get cool worm-like patterns filling
// up the sketch.  If you really want more information, try these
// links:

// http://www.karlsims.com/rd.html
// http://mrob.com/pub/comp/xmorphia/
// http://en.wikipedia.org/wiki/Reaction%E2%80%93diffusion_system

// Here's a really nicely implemented version that runs in
// your browser with GPU acceleration:
// https://pmneila.github.io/jsexp/grayscott/

float[] A1;
float[] B1;

float[] A2;
float[] B2;

float DA;
float DB;
float f;
float k;
float dT;

float amin;
float amax;

PImage img;

float[] kern = { 0.05, 0.2, 0.05, 0.2, -1, 0.2, 0.05, 0.2, 0.05 };

void setup()
{
  size( 256, 256 );
  A1 = new float[256*256];
  B1 = new float[256*256];
  A2 = new float[256*256];
  B2 = new float[256*256];
  
  DA = 1.0;
  DB = 0.5;
  f = 0.055;
  k = 0.062;
  dT = 1.0;
      
  img = createImage( 256, 256, RGB );
  
  initGrids();
}
  
void initGrids()
{
  for( int y = 0; y < 256; ++y ) {
    for( int x = 0; x < 256; ++x ) {
      A1[y*256+x] = 1.0;
      B1[y*256+x] = noise(x*0.02,y*0.02) < 0.55 ? 1 : 0;
     
      /*
      if( x > 100 && x < 120 && y > 100 && y < 120 ) {
        B1[y*256+x] = 1.0;
      }
      */
    }
  }
}

float get( float[] A, int x, int y )
{
  int xx = (x+256)%256;
  int yy = (y+256)%256;
  return A[yy*256+xx];
}

float conv3( float[] A, int x, int y, float[] weights )
{
  float tot = 0.0;
  
  for( int dy = 0; dy < 3; ++dy ) {
    for( int dx = 0; dx < 3; ++dx ) {
      tot += weights[dy*3+dx]*get(A,x+dx-1,y+dy-1);
    }
  }
  return tot;
}

void step()
{
  amin = 1.0;
  amax = 0.0;
  
  for( int y = 0; y < 256; ++y ) {
    for( int x = 0; x < 256; ++x ) {
      float A = get( A1, x, y );
      float B = get( B1, x, y );
      float LA = conv3( A1, x, y, kern );
      float LB = conv3( B1, x, y, kern );
      
      float a2 = A+(DA*LA-A*B*B+f*(1-A))*dT;
      float b2 = B+(DB*LB+A*B*B-(k+f)*B)*dT;
      
      amin = min( a2, amin );
      amax = max( a2, amax );

      A2[y*256+x] = a2;
      B2[y*256+x] = b2;
    }
  }
  
  float[] atmp = A1;
  float[] btmp = B1;
  
  A1 = A2;
  B1 = B2;
  
  A2 = atmp;
  B2 = btmp;
}

void draw()
{
  // println( frameCount );
  noFill();
  for( int idx = 0; idx < 20; ++idx ) {
    step();
  }
  background( 255 );
  
  img.loadPixels();
  
  for( int y = 0; y < 256; ++y ) {
    for( int x = 0; x < 256; ++x ) {
      float A = A1[y*256+x];
      int g = int(norm(A,amin,amax)*255);
     
      img.pixels[256*y+x] = color(g,g,g);
    }
  }
  
  img.updatePixels();
  image( img, 0, 0 );
}

void keyPressed()
{
  initGrids();
  f = random( 0.05, 0.07 );
  k = random( 0.05, 0.08 );
}
