// Draw a fractal mountain based on subdivision.  Repeatedly
// subdivide a curve horizontally and generate a new height
// for this horizontal location by offsetting away from the 
// original height by some random value.

import processing.pdf.*;

float init = 50.0;
float scaling = 0.5;
int seed = 0;
int levels = 7;
boolean record = false;

boolean do_gaussian = true;

void drawMountain(
  int levs, float x1, float y1, float x2, float y2 )
{
  if ( levs == 0 ) {
    // Base case: just add a single point to the mountain
    // curve.
    
    vertex( x2, y2 );
  } else {
    // Recursive case: break the problem down in the left
    // half of this line and the right half of the line by
    // computing the current midpoint.  Move the midpoint
    // up or down by some random amount.  Then recursively
    // draw slightly simpler mountains that join the two
    // resulting line segments.
    
    float M = scaling * (x2 - x1);
    float xmid = 0.5*(x1+x2);
    float ymid = 0.5*(y1+y2);
    float r = 0.0;
    
    // Just for the purposes of experimentation, try both
    // Gaussian random numbers and uniform (regular) random
    // numbers.  Don't worry about this -- I won't ever ask
    // you about randomGaussian().
    if( do_gaussian ) {
      r = randomGaussian()*M;
    } else {
      r = random( -M, M );
    }
    
    ymid += r;
    drawMountain( levs - 1, x1, y1, xmid, ymid );
    drawMountain( levs - 1, xmid, ymid, x2, y2 );
  }
}

void setup()
{
  size( 600, 400 );
}

void draw()
{
  if( record ) {
    println( "Recording." );
    beginRecord( PDF, "output.pdf" );
  }
  
  // Use the trick of setting the random seed to some initial
  // value, guaranteeing that we can "play back" the same 
  // mountain from frame to frame and get some smooth behaviour.
  randomSeed( seed );
  
  scaling = map( mouseX, 0, width, 0, 1 );
  
  background( #a0c0ff );
  fill( #106000 );
  beginShape();
  vertex( 0, height );
  float y1 = random( 200, 300 );
  float y2 = random( 200, 300 );
  vertex( 0, y1 );
  drawMountain( levels, 0, y1, width, y2 );
  vertex( width, height );
  endShape( CLOSE );
  
  if( record ) {
    endRecord();
    record = false;
  }
}

void keyPressed()
{
  if ( key == ' ' ) {
    seed = millis();
  } else if ( key == '-' ) {
    levels = max( 0, levels - 1 );
  } else if( (key == '=') || (key == '+') ) {
    ++levels;
  } else if( key == 'g' ) {
    do_gaussian = !do_gaussian;
  } else if( key == 'p' ) {
    record = true;
  }
}

